﻿Imports System
Imports System.Runtime.InteropServices


Namespace MediaInfoLib

    Public Enum StreamKind
        General
        Video
        Audio
        Text
        Other
        Image
        Menu
    End Enum

    Public Enum InfoKind
        Name
        Text
        Measure
        Options
        NameText
        MeasureText
        Info
        HowTo
    End Enum

    Public Enum InfoOptions
        ShowInInform
        Support
        ShowInSupported
        TypeOfValue
    End Enum

    Public Enum InfoFileOptions
        FileOption_Nothing = &H0
        FileOption_NoRecursive = &H1
        FileOption_CloseAll = &H2
        FileOption_Max = &H4
    End Enum

    Public Enum Status
        None = &H0
        Accepted = &H1
        Filled = &H2
        Updated = &H4
        Finalized = &H8
    End Enum

    Public Class MediaInfo

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_New() As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Sub MediaInfo_Delete(Handle As IntPtr)
        End Sub

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_Open(Handle As IntPtr, <MarshalAs(UnmanagedType.LPWStr)> FileName As String) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoA_Open(Handle As IntPtr, FileName As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_Open_Buffer_Init(Handle As IntPtr, File_Size As Int64, File_Offset As Int64) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoA_Open(Handle As IntPtr, File_Size As Int64, File_Offset As Int64) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_Open_Buffer_Continue(Handle As IntPtr, Buffer As IntPtr, Buffer_Size As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoA_Open_Buffer_Continue(Handle As IntPtr, File_Size As Int64, Buffer As Byte(), Buffer_Size As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_Open_Buffer_Continue_GoTo_Get(Handle As IntPtr) As Int64
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoA_Open_Buffer_Continue_GoTo_Get(Handle As IntPtr) As Int64
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_Open_Buffer_Finalize(Handle As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoA_Open_Buffer_Finalize(Handle As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Sub MediaInfo_Close(Handle As IntPtr)
        End Sub

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_Inform(Handle As IntPtr, Reserved As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoA_Inform(Handle As IntPtr, Reserved As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_GetI(Handle As IntPtr, StreamKind As IntPtr, StreamNumber As IntPtr, Parameter As IntPtr, KindOfInfo As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoA_GetI(Handle As IntPtr, StreamKind As IntPtr, StreamNumber As IntPtr, Parameter As IntPtr, KindOfInfo As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_Get(Handle As IntPtr, StreamKind As IntPtr, StreamNumber As IntPtr, <MarshalAs(UnmanagedType.LPWStr)> Parameter As String, KindOfInfo As IntPtr, KindOfSearch As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoA_Get(Handle As IntPtr, StreamKind As IntPtr, StreamNumber As IntPtr, Parameter As IntPtr, KindOfInfo As IntPtr, KindOfSearch As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_Option(Handle As IntPtr, <MarshalAs(UnmanagedType.LPWStr)> [Option] As String, <MarshalAs(UnmanagedType.LPWStr)> Value As String) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoA_Option(Handle As IntPtr, [Option] As IntPtr, Value As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_State_Get(Handle As IntPtr) As IntPtr
        End Function

        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfo_Count_Get(Handle As IntPtr, StreamKind As IntPtr, StreamNumber As IntPtr) As IntPtr
        End Function

        Private Handle As IntPtr
        Private MustUseAnsi As Boolean

        Public Sub New()
            Try
                Handle = MediaInfo_New()
            Catch
                Handle = CType(0, IntPtr)
            End Try
            If Environment.OSVersion.ToString().IndexOf("Windows") = -1 Then
                MustUseAnsi = True
            Else
                MustUseAnsi = False
            End If
        End Sub

        Protected Overrides Sub Finalize()
            Try
                If Handle = CType(0, IntPtr) Then
                    Return
                End If
                MediaInfo_Delete(Handle)
            Finally
                MyBase.Finalize()
            End Try
        End Sub

        Public Function Open(FileName As [String]) As Integer
            If Handle = CType(0, IntPtr) Then
                Return 0
            End If
            If MustUseAnsi Then
                Dim FileName_Ptr As IntPtr = Marshal.StringToHGlobalAnsi(FileName)
                Dim ToReturn As Integer = CInt(MediaInfoA_Open(Handle, FileName_Ptr))
                Marshal.FreeHGlobal(FileName_Ptr)
                Return ToReturn
            Else
                Return CInt(MediaInfo_Open(Handle, FileName))
            End If
        End Function

        Public Function Open_Buffer_Init(File_Size As Int64, File_Offset As Int64) As Integer
            If Handle = CType(0, IntPtr) Then
                Return 0
            End If
            Return CInt(MediaInfo_Open_Buffer_Init(Handle, File_Size, File_Offset))
        End Function
        Public Function Open_Buffer_Continue(Buffer As IntPtr, Buffer_Size As IntPtr) As Integer
            If Handle = CType(0, IntPtr) Then
                Return 0
            End If
            Return CInt(MediaInfo_Open_Buffer_Continue(Handle, Buffer, Buffer_Size))
        End Function
        Public Function Open_Buffer_Continue_GoTo_Get() As Int64
            If Handle = CType(0, IntPtr) Then
                Return 0
            End If
            Return CType(MediaInfo_Open_Buffer_Continue_GoTo_Get(Handle), Int64)
        End Function
        Public Function Open_Buffer_Finalize() As Integer
            If Handle = CType(0, IntPtr) Then
                Return 0
            End If
            Return CInt(MediaInfo_Open_Buffer_Finalize(Handle))
        End Function
        Public Sub Close()
            If Handle = CType(0, IntPtr) Then
                Return
            End If
            MediaInfo_Close(Handle)
        End Sub

        Public Function Inform() As [String]
            If Handle = CType(0, IntPtr) Then
                Return "Unable to load MediaInfo library"
            End If
            If MustUseAnsi Then
                Return Marshal.PtrToStringAnsi(MediaInfoA_Inform(Handle, CType(0, IntPtr)))
            Else
                Return Marshal.PtrToStringUni(MediaInfo_Inform(Handle, CType(0, IntPtr)))
            End If
        End Function
        Public Function [Get](StreamKind As StreamKind, StreamNumber As Integer, Parameter As [String], KindOfInfo As InfoKind, KindOfSearch As InfoKind) As [String]
            If Handle = CType(0, IntPtr) Then
                Return "Unable to load MediaInfo library"
            End If
            If MustUseAnsi Then
                Dim Parameter_Ptr As IntPtr = Marshal.StringToHGlobalAnsi(Parameter)
                Dim ToReturn As [String] = Marshal.PtrToStringAnsi(MediaInfoA_Get(Handle, CType(StreamKind, IntPtr), CType(StreamNumber, IntPtr), Parameter_Ptr, CType(KindOfInfo, IntPtr), CType(KindOfSearch, IntPtr)))
                Marshal.FreeHGlobal(Parameter_Ptr)
                Return ToReturn
            Else
                Return Marshal.PtrToStringUni(MediaInfo_Get(Handle, CType(StreamKind, IntPtr), CType(StreamNumber, IntPtr), Parameter, CType(KindOfInfo, IntPtr), CType(KindOfSearch, IntPtr)))
            End If
        End Function
        Public Function [Get](StreamKind As StreamKind, StreamNumber As Integer, Parameter As Integer, KindOfInfo As InfoKind) As [String]
            If Handle = CType(0, IntPtr) Then
                Return "Unable to load MediaInfo library"
            End If
            If MustUseAnsi Then
                Return Marshal.PtrToStringAnsi(MediaInfoA_GetI(Handle, CType(StreamKind, IntPtr), CType(StreamNumber, IntPtr), CType(Parameter, IntPtr), CType(KindOfInfo, IntPtr)))
            Else
                Return Marshal.PtrToStringUni(MediaInfo_GetI(Handle, CType(StreamKind, IntPtr), CType(StreamNumber, IntPtr), CType(Parameter, IntPtr), CType(KindOfInfo, IntPtr)))
            End If
        End Function


        Public Function [Option]([Option_] As [String], Value As [String]) As [String]
            If Handle = CType(0, IntPtr) Then
                Return "Unable to load MediaInfo library"
            End If
            If MustUseAnsi Then
                Dim Option_Ptr As IntPtr = Marshal.StringToHGlobalAnsi([Option_])
                Dim Value_Ptr As IntPtr = Marshal.StringToHGlobalAnsi(Value)
                Dim ToReturn As [String] = Marshal.PtrToStringAnsi(MediaInfoA_Option(Handle, Option_Ptr, Value_Ptr))
                Marshal.FreeHGlobal(Option_Ptr)
                Marshal.FreeHGlobal(Value_Ptr)
                Return ToReturn
            Else
                Return Marshal.PtrToStringUni(MediaInfo_Option(Handle, [Option_], Value))
            End If
        End Function

        Public Function State_Get() As Integer
            If Handle = CType(0, IntPtr) Then
                Return 0
            End If
            Return CInt(MediaInfo_State_Get(Handle))
        End Function

        Public Function Count_Get(StreamKind As StreamKind, StreamNumber As Integer) As Integer
            If Handle = CType(0, IntPtr) Then
                Return 0
            End If
            Return CInt(MediaInfo_Count_Get(Handle, CType(StreamKind, IntPtr), CType(StreamNumber, IntPtr)))
        End Function

        Public Function [Get](StreamKind As StreamKind, StreamNumber As Integer, Parameter As [String], KindOfInfo As InfoKind) As [String]
            Return [Get](StreamKind, StreamNumber, Parameter, KindOfInfo, InfoKind.Name)
        End Function
        Public Function [Get](StreamKind As StreamKind, StreamNumber As Integer, Parameter As [String]) As [String]
            Return [Get](StreamKind, StreamNumber, Parameter, InfoKind.Text, InfoKind.Name)
        End Function
        Public Function [Get](StreamKind As StreamKind, StreamNumber As Integer, Parameter As Integer) As [String]
            Return [Get](StreamKind, StreamNumber, Parameter, InfoKind.Text)
        End Function
        Public Function [Option](Option_ As [String]) As [String]
            Return [Option](Option_, "")
        End Function
        Public Function Count_Get(StreamKind As StreamKind) As Integer
            Return Count_Get(StreamKind, -1)
        End Function

    End Class


    Public Class MediaInfoList
        'Import of DLL functions. DO NOT USE until you know what you do (MediaInfo DLL do NOT use CoTaskMemAlloc to allocate memory)
        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoList_New() As IntPtr
        End Function
        <DllImport("MediaInfo.dll")> _
        Private Shared Sub MediaInfoList_Delete(Handle As IntPtr)
        End Sub
        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoList_Open(Handle As IntPtr, <MarshalAs(UnmanagedType.LPWStr)> FileName As String, Options As IntPtr) As IntPtr
        End Function
        <DllImport("MediaInfo.dll")> _
        Private Shared Sub MediaInfoList_Close(Handle As IntPtr, FilePos As IntPtr)
        End Sub
        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoList_Inform(Handle As IntPtr, FilePos As IntPtr, Reserved As IntPtr) As IntPtr
        End Function
        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoList_GetI(Handle As IntPtr, FilePos As IntPtr, StreamKind As IntPtr, StreamNumber As IntPtr, Parameter As IntPtr, KindOfInfo As IntPtr) As IntPtr
        End Function
        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoList_Get(Handle As IntPtr, FilePos As IntPtr, StreamKind As IntPtr, StreamNumber As IntPtr, <MarshalAs(UnmanagedType.LPWStr)> Parameter As String, KindOfInfo As IntPtr, _
        KindOfSearch As IntPtr) As IntPtr
        End Function
        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoList_Option(Handle As IntPtr, <MarshalAs(UnmanagedType.LPWStr)> [Option] As String, <MarshalAs(UnmanagedType.LPWStr)> Value As String) As IntPtr
        End Function
        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoList_State_Get(Handle As IntPtr) As IntPtr
        End Function
        <DllImport("MediaInfo.dll")> _
        Private Shared Function MediaInfoList_Count_Get(Handle As IntPtr, FilePos As IntPtr, StreamKind As IntPtr, StreamNumber As IntPtr) As IntPtr
        End Function

        'MediaInfo class
        Public Sub New()
            Handle = MediaInfoList_New()
        End Sub
        Protected Overrides Sub Finalize()
            Try
                MediaInfoList_Delete(Handle)
            Finally
                MyBase.Finalize()
            End Try
        End Sub
        Public Function Open(FileName As [String], Options As InfoFileOptions) As Integer
            Return CInt(MediaInfoList_Open(Handle, FileName, CType(Options, IntPtr)))
        End Function
        Public Sub Close(FilePos As Integer)
            MediaInfoList_Close(Handle, CType(FilePos, IntPtr))
        End Sub
        Public Function Inform(FilePos As Integer) As [String]
            Return Marshal.PtrToStringUni(MediaInfoList_Inform(Handle, CType(FilePos, IntPtr), CType(0, IntPtr)))
        End Function
        Public Function [Get](FilePos As Integer, StreamKind As StreamKind, StreamNumber As Integer, Parameter As [String], KindOfInfo As InfoKind, KindOfSearch As InfoKind) As [String]
            Return Marshal.PtrToStringUni(MediaInfoList_Get(Handle, CType(FilePos, IntPtr), CType(StreamKind, IntPtr), CType(StreamNumber, IntPtr), Parameter, CType(KindOfInfo, IntPtr), _
                CType(KindOfSearch, IntPtr)))
        End Function
        Public Function [Get](FilePos As Integer, StreamKind As StreamKind, StreamNumber As Integer, Parameter As Integer, KindOfInfo As InfoKind) As [String]
            Return Marshal.PtrToStringUni(MediaInfoList_GetI(Handle, CType(FilePos, IntPtr), CType(StreamKind, IntPtr), CType(StreamNumber, IntPtr), CType(Parameter, IntPtr), CType(KindOfInfo, IntPtr)))
        End Function
        Public Function [Option]([Option_] As [String], Value As [String]) As [String]
            Return Marshal.PtrToStringUni(MediaInfoList_Option(Handle, [Option_], Value))
        End Function
        Public Function State_Get() As Integer
            Return CInt(MediaInfoList_State_Get(Handle))
        End Function
        Public Function Count_Get(FilePos As Integer, StreamKind As StreamKind, StreamNumber As Integer) As Integer
            Return CInt(MediaInfoList_Count_Get(Handle, CType(FilePos, IntPtr), CType(StreamKind, IntPtr), CType(StreamNumber, IntPtr)))
        End Function
        Private Handle As IntPtr

        'Default values, if you know how to set default values in C#, say me
        Public Function Open(FileName As [String]) As Integer
            Return Open(FileName, 0)
        End Function
        Public Sub Close()
            Close(-1)
        End Sub
        Public Function [Get](FilePos As Integer, StreamKind As StreamKind, StreamNumber As Integer, Parameter As [String], KindOfInfo As InfoKind) As [String]
            Return [Get](FilePos, StreamKind, StreamNumber, Parameter, KindOfInfo, InfoKind.Name)
        End Function
        Public Function [Get](FilePos As Integer, StreamKind As StreamKind, StreamNumber As Integer, Parameter As [String]) As [String]
            Return [Get](FilePos, StreamKind, StreamNumber, Parameter, InfoKind.Text, InfoKind.Name)
        End Function
        Public Function [Get](FilePos As Integer, StreamKind As StreamKind, StreamNumber As Integer, Parameter As Integer) As [String]
            Return [Get](FilePos, StreamKind, StreamNumber, Parameter, InfoKind.Text)
        End Function
        Public Function [Option](Option_ As [String]) As [String]
            Return [Option](Option_, "")
        End Function
        Public Function Count_Get(FilePos As Integer, StreamKind As StreamKind) As Integer
            Return Count_Get(FilePos, StreamKind, -1)
        End Function
    End Class


End Namespace

